# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################

"""classificateur bayesien

Ce classificateur 

- collecte des mots clefs à partir des répertoires de données
 (par exemple :  *porn* et *noporn*)
- renvoie un coefficient sur la page web ou le contenu parsé

guesser.train('french', 'le la les du un une je il elle de en')

fonctionnement du filtrage bayesien

une phase d'apprentissage 

la base de données de filtrage est fabriquée à partir du contenu de fichiers dans un
répertoire portant le nom du mot-clef (thème de filtrage)

exemple : des répertoires `porn` et `noporn` contenant des fichiers html avec des contenus correspondant aux cibles des mots-clefs permettent un apprentissage

la phase de notation

si un contenu s'approche d'un mot clef, alors le mot clef est affiché
avec un certain pourcentage qui correspond à une évaluation. 

plusieurs mots-clefs peuvent apparaître si un contenu correspond à
plusieurs cible à la fois, avec chacune un coefficient associé à
chaque mot-clef. 

"""

from os import path as osp
from glob import glob
import re
from sets import Set
from reverend.thomas import Bayes

from filtrage.config import (GUESS_DATA, BAYES_DATABASE,
                             KEYWORDS, TRAIN_MODE, MINIMAL_CONTENT)
from filtrage.errors import DataError, MinimalContentError
from filtrage.tools import filter_meta, beautiful_soup, normalize_string, average
from filtrage.db import load_pickle


def filter_line(string, mode):
    if mode == 'meta':
        content = filter_meta(string)
    elif mode == 'html':
        content = beautiful_soup(string)
    else:
        # mode txt
        content = normalize_string(string)
    # le contenu à tester est trop petit
    if len(content.split()) <= MINIMAL_CONTENT:
        raise MinimalContentError
    return content


class StatisticsGuesser:

    def  __init__(self, filename='filtrage.dat'):
        """
        filename: nom long du fichier bayesien
        """
        self.filename = filename
        # dico des mots clefs et de leur ponderation
        self.bayesdict = self._load()
        self.comp_bayesdict = self._comp_bayesdict()

    def _load(self, filename=None):
        """charge un fichier bayesien
        construit le dictionnaire des mots clefs
        avec leurs ponderations respectives
        {mot_clef: (ponderation_porn, ponderation_noporn)}
        {'cul':(8, 0)}
        """
        if filename:
            self.filename = filename
        try:
            return load_pickle(self.filename)
            
        except:
            raise DataError, "le fichier de donnees de filtrage est inexistant ou bien corrompu"

    def _comp_bayesdict(self):
        """intersection des valeurs de dictionnaire porn et noporn"""
        porn_set = Set(self.bayesdict['porn'].keys())
        noporn_set = Set(self.bayesdict['noporn'].keys())
        compare_result = porn_set.intersection(noporn_set)
        comp_bayesdict = {}
        for keyword in compare_result:
            comp_bayesdict[keyword] = (self.bayesdict['porn'][keyword],
                                      self.bayesdict['noporn'][keyword])
        return comp_bayesdict


    def get_proba(self, proba_dict):
        l = []
        for key, value in proba_dict.items():
            l.append(self.get_simple_proba(*value))
        return sum(l)/len(l)
        
    def get_simple_proba(self, porn, noporn):
        return 1.0*porn/(porn+noporn)


        
    def guess_line(self, string, mode='meta'):
        # XXFIXME : il faudra compter les occurences
        content = filter_line(string, mode)
        content_keywords = []
        for keyword in self.comp_bayesdict.keys():
            if keyword in content:
                content_keywords.append(keyword)

        # contruction du dictionnaire de proba
        content_proba_dict = {}
        for keyword in content_keywords:
            content_proba_dict[keyword] = self.comp_bayesdict[keyword]

        return self.get_proba(content_proba_dict)

    # XXFIXME gérer le cas 'weighted'
#             guess_meta = self.guesser.guess(filter_meta(string))
#             guess_body = self.guesser.guess(beautiful_soup(string))
        


class Guesser:
    def  __init__(self, filename='guesser.dat'):
        """
        filename: nom long du fichier bayesien
        """
        self.guesser = Bayes()
        self.filename = filename

    def train(self, dirname, keyword, mode='meta'):
        """charge tous les fichiers d'un repertoire
        :dirname: nom du repertoire
        :keyword: mot clef associe a ce repertoire"""

        dirname = osp.abspath(osp.normpath(osp.expanduser(dirname)))

        if not osp.isdir(dirname):
            raise Exception, "un repertoire est requis, %s" % dirname

        for filename in glob(osp.join(dirname, "*.*")):
            fh = file(filename, 'r')
            content = fh.read()

            # les différents modes de filtrage possibles
            if mode == 'meta':
                content = filter_meta(content)
            elif mode == 'html':
                content = beautiful_soup(content)
            else:
                # mode txt
                content = normalize_string(content)

            self.guesser.train(keyword, content)
            fh.close()

    def save(self, filename=None):
        """enregistrement de la base bayesienne
        """
        if not filename:
            filename = self.filename
        self.guesser.save(filename)

    def load(self, filename=None):
        "charge un fichier bayesien (en extension .bay)"
        if filename:
            self.filename = filename
        try:
            self.guesser.load(self.filename)
        except:
            raise DataError, "le fichier de donnees de filtrage est inexistant ou bien corrompu"
        
    def guess_line(self, string, mode='meta'):
        """affecte une note de filtrage à la ligne string
        suivant le mode choisi"""
        if mode == 'weight':
            # FIXME : pour l'instant, seul le body et le meta 
            guess_meta = self.guesser.guess(filter_meta(string))
            guess_body = self.guesser.guess(beautiful_soup(string))
            return average(guess_meta, guess_body)
        else:
            content = filter_line(string, mode)

        return self.guesser.guess(content)

#     def guess_file(self, filename):
#         """ouvre un fichier et teste son contenu"""
#         filename = osp.abspath(osp.normpath(osp.expanduser(filename)))
#         if not osp.isfile(filename):
#             raise Exception, "nom de fichier : %s" % filename
#         fh = file(filename, 'r')
#         string = fh.read()
#         fh.close()
#         return self.guess_line(string)

def train_guesser(keywords=KEYWORDS, filename=BAYES_DATABASE, guess_data=GUESS_DATA,
                  mode=TRAIN_MODE):
    """fabrique le classificateur suivant les données
    de configuration et l'enregistre
    chaque nom de répertoire doit avoir le même nom que le nom des mots clefs
    """
    # entrainement sur tous les mot-clefs
    g = Guesser(filename=filename)
    print "creation de la base..."
    for key in keywords:
        print "le mot clef est : ", key
        # creation des bases
        print osp.join(guess_data, key)
        g.train(osp.join(guess_data, key), key, mode=mode)

    # enregistrement de la base 
    g.save(filename)


if __name__ == '__main__':
    
    stat = StatisticsGuesser(filename ='../data/filtrage.dat')

    print stat.guess_line("""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd"> <html> <head> <meta http-equiv="Content-Language" content="en-us"> <title>Adult Search Engine - Where The Fuck</title> <meta name="revisit-after" content="15 days"> <meta name="classification" content="Entertainment"> <meta name="keywords" content="free adult search engine, adult search engine, adult content search engine, hottest adult search engine, adult movie search engine, adult sex search engine, adult image search engine, uk adult search engine, yahoo adult search engine, free adult search engine, adult porn search engine, adult site search engine, adult engine search underground, adult xxx search engine, adult internet search engine, adult video search engine"> <meta name="description" content="sex guide featuring adult websites related to xxx images, porn movies, sexual health, and more. Reviewed and rated by you."> <meta name="distribution" content="Global"> <meta name="rating" content="Mature"> <meta name="copyright" content="2002. Demolition Crew Inc."> <meta name="language" content="en-us"> <meta http-equiv="pics-label" content="(pics-1.1 &quot;http://www.icra.org/ratingsv02.html&quot; comment &quot;ICRAonline v2.0&quot; l gen true for &quot;http://www.wherethefuck.com&quot;  r (ng 1 vz 1 la 1 lb 1 lc 1 og 1 cz 1) &quot;http://www.rsac.org/ratingsv01.html&quot; l gen true for &quot;http://www.wherethefuck.com&quot;  r (n 0 s 3 v 0 l 4))"> <meta name="robots" content="index,follow"> <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"> <style type="TEXT/CSS"> <!--
A { text-decoration: none }
A {color:#000099; font-family: Helvetica, Arial; font-size: 12px; text-decoration: none;}
A:hover {color:#000000; font-family: Helvetica, Arial; font-size: 12px; text-decoration: none;}
td {font-family: Helvetica, Arial; font-size: 11px;}
.linkcell { padding-left: 10px; font-family: Helvetica, Arial; font-size: 10px;}
.button { background-color: #CCCCCC; layer-background-color: #CCCCCC; border: 1px solid #333333; font-family:Verdana,Arial; font-size: 10px; font-weight:bold; }
hr {  color: #000000}
//.unnamed1 {  font-family: Arial, Helvetica, sans-serif; font-size: small}
--> </style> <script language="JavaScript"> <!--
function MM_preloadImages() { //v3.0
 var d=document; if(d.images){ if(!d.MM_p) d.MM_p=new Array();
 var i,j=d.MM_p.length,a=MM_preloadImages.arguments; for(i=0; i<a.length; i++)
 if (a[i].indexOf("#")!=0){ d.MM_p[j]=new Image; d.MM_p[j++].src=a[i];}}
}
//--> </script> </head> <body bgcolor="#FFFFFF" leftmargin=0 topmargin=0 marginwidth=0 marginheight=0 onLoad="MM_preloadImages('http://www.wherethefuck.com/images/button_login_r.gif')" background="http://www.wherethefuck.com/images/bg.gif"> <table width=740 border=0 cellpadding=0 cellspacing=0 align="left" height="712"> <tr> <td rowspan=8 height="712"> <img src="images/left_white.gif" width=28 height=540 alt=""></td> <td colspan=2 height="11"> <img src="images/top.gif" width=691 height=11 alt=""></td> <td rowspan=8 height="712"> <img src="images/right_white.gif" width=21 height=540 alt=""></td> </tr> <tr> <td rowspan="4" valign="top" bgcolor="333333" height="239"> <table width="100%" border="0" cellspacing="0" cellpadding="0" style="border-collapse: collapse" bordercolor="#111111" bgcolor="#333333"> <tr> <td background="http://www.wherethefuck.com/images/bg.gif"> <img src="images/top_curve.gif" width=201 height=27 alt=""></td> </tr> <tr> <td valign="top"> <table width="100%" border="0" cellspacing="0" cellpadding="0" bgcolor="#333333" style="border-collapse: collapse" bordercolor="#111111"> <tr> <td width="201" height="59" bgcolor="#333333"> <img border="0" src="http://www.wherethefuck.com/images/gray.gif" width="201" height="81"></td> </tr> <tr> <td valign="baseline" bgcolor="#333333"> <table width="100%" border="0" cellspacing="2" cellpadding="2" align="center"> <tr bgcolor="#000000"> <td> <div align="center"><font color="#FFFFFF"><b>Begin
 Your Adult Search Here</b></font></div></td> </tr> <tr> <td> <form action="http://www.wherethefuck.com/cgi-bin/search.cgi" method="GET"> <div align="right">&nbsp; <br> <input class="button" type="text" name="query" size="20"><br> <input class="button" type="submit" value="Go">
 &nbsp;<br> </div> </form></td> </tr> <tr> <td width="100%"> <p align="center"> <font color="#FFCC00">
 10103
 links listed as of <br>
 Tue Apr 08 2003
</font> </td> </tr> </table></td> </tr> </table></td> </tr> </table></td> <td height="27"> <img src="http://www.wherethefuck.com/images/home_image1.jpg" width=490 height=27 alt=""></td> </tr> <tr> <td height="59"> <table width="100%" border="0" cellspacing="0" cellpadding="0" height="59"> <tr> <td background="http://www.wherethefuck.com/images/home_image2.jpg"> <div align="right">&nbsp;<font size="4">&nbsp;<font color="#FFFFFF"><b>WWW<font color="#000000">.WHERETHEFUCK.COM</font></b></font></font><font size="5">&nbsp;</font></div></td> </tr> </table></td> </tr> <tr> <td height="120"> <table width="100%" border="0" cellspacing="0" cellpadding="0" height="120"> <tr> <td background="http://www.wherethefuck.com/images/home_image3.jpg" valign="top" height="96">&nbsp;
</td> </tr> </table></td> </tr> <tr> <td height="33" bgcolor="#000000"> <table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#000000"> <tr> <td bgcolor="#000000"> <div align="center"><small> <a style="color: #FFFFFF" href="http://www.wherethefuck.com/New">
 &nbsp;NEW</a> &nbsp;<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a style="color: #FFFFFF" href="http://www.wherethefuck.com/Cool"> &nbsp;COOL</a>
 &nbsp;<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a style="color: #FFFFFF" href="http://www.wherethefuck.com/Ratings"> &nbsp;TOP
 RATED</a>&nbsp;<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine">
 &nbsp; <a style="color: #FFFFFF" href="http://www.wherethefuck.com/cgi-bin/gforums/gforum.cgi?">SUPPORT</a>&nbsp;<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a style="color: #FFFFFF" href="http://www.wherethefuck.com/advertise.html">&nbsp;ADVERTISE</a>&nbsp;
<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a style="color: #FFFFFF" href="http://www.wherethefuck.com/add.html">&nbsp;ADD
 A SITE</a></small></div></td> </tr> </table></td> </tr> <tr> <td height="463" colspan=2 bordercolor="#000000" bgcolor="#000000"> <table width="100%" height="275" border="0" cellpadding="0" cellspacing="0" bordercolor="#000000" bgcolor="#000000"> <tr> <td background="http://www.wherethefuck.com/images/text_area1.gif" valign="top"> <table width="100%" border="0" cellspacing="0" cellpadding="0"> <tr> <td width="29%" height="100%" valign="top" bgcolor="#999999"> <table width="95%" height="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#999999"> <tr> <td bgcolor="#999999" valign="top"> <table width="100%" height="100%" border="0" cellpadding="0" cellspacing="0"> <tr> <td width="100%" valign="top" bgcolor="#000000"> <div align="center"><strong><font color="#FFFFFF">-Welcome
 to
 Where
 The
 Fuck
 -</font> </strong></div></td> </tr> <tr> <td width="100%" align="left" valign="top">&nbsp;
</td> </tr> <tr> <td width="100%" align="left" valign="top"> <p><font face="Arial" size="2">Welcome
 to
 our
 adult
 search
 engine.
 Stop
 looking
 in
 all
 the
 wrong
 places
 to
 find
 what
 you
 want
 --
 or
 need
 as
 the
 case
 may
 be.&nbsp;
 Our
 powerful
 adult
 search
 engine
 will
 find
 whatever
 the
 fuck
 you
 want
 it
 to.</font></p> <p><font face="Arial" size="2">Don't
 get
 fucked
 by
 spending
 all
 of
 your
 valuable
 time
 aimlessly
 looking
 for
 quality
 adult
 porn
 and
 products
 when
 you
 can
 find
 it
 quickly
 and
 easily
 right
 here.</font></p> <p><font face="Arial" size="2">If
 you
 find
 a
 site
 that
 is
 &quot;the
 fuck&quot;,
 then
 you
 can
 rate
 it
 and
 let
 other
 people
 know
 about
 it.&nbsp;
 If
 a
 site
 &quot;fucks
 you
 over&quot;,
 then
 you
 will
 certainly
 want
 to
 let
 others
 know.&nbsp;
 That
 is
 why
 this
 Adult
 Search
 Engine
 truly
 kicks
 the
 shit
 out
 of
 the
 other
 adult
 directories
 on
 the
 web!</font></p> <p><font face="Arial" size="2">In
 closing,
 we
 should
 say
 that
 if
 you
 don't
 want
 to
 go
 to
 sites
 and
 see
 hardcore
 sex
 photos,
 xxx
 rated
 adult
 videos,
 or
 even
 some
 kinky
 shit
 that
 we
 won't
 go
 into
 here
 on
 the
 front
 page,
 then
 we
 politely
 ask
 you
 to
 please
 get
 the
 fuck
 out.</font></p> </td> </tr> </table></td> </tr> </table> <td width="71%" height="282" valign="top"> <table width="100%" border="0" cellpadding="2" cellspacing="2" bgcolor="#000000"> <tr> <td colspan="2" valign="top" bgcolor="#000000"> <p align="left"><table width="480" border="0" align="center" cellpadding="2" cellspacing="0" bgcolor="#990066"> <tr> <td colspan="4" bgcolor="#000000"><div align="center"><strong><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><font color="#FFFFFF" size="2" face="Verdana, Arial, Helvetica, sans-serif">THE LARGEST HOME WEBCAM NETWORK ONLINE!</font></a></strong></div></td> </tr> <tr> <td colspan="4"><div align="center"><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><img src="/images/girlcams.jpg" alt="HOME WEB CAM COMMUNITY" width="480" height="90" vspace="3" border="0"></a></div></td> </tr> <tr> <td width="25%"><div align="center"><strong><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><font color="#FFFF00" size="2" face="Verdana, Arial, Helvetica, sans-serif">OrientGirl18</font></a></strong></div></td> <td width="25%"><div align="center"><strong><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><font color="#FFFF00" size="2" face="Verdana, Arial, Helvetica, sans-serif">Gift4You</font></a></strong></div></td> <td width="25%"><div align="center"><strong><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><font color="#FFFF00" size="2" face="Verdana, Arial, Helvetica, sans-serif">CheakyGirl</font></a></strong></div></td> <td width="25%"><div align="center"><strong><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><font color="#FFFF00" size="2" face="Verdana, Arial, Helvetica, sans-serif">Samantha18</font></a></strong></div></td> </tr> <tr> <td colspan="4" bgcolor="#000000"><div align="center"><strong><a href="http://www.free-webcam-shows.com/camgirls/index.htm"><font color="#FFFFFF" size="2" face="Verdana, Arial, Helvetica, sans-serif">CLICK HERE TO SEE ALL OF THE HOT GIRLS BROADCATING NOW</font></a></strong></div></td> </tr> </table> </td> </tr> <tr> <td valign="top" bgcolor="#FFFFFF"> <p align="left"><font color="#000000" size="2" face="Arial, Helvetica, sans-serif"><strong>Great
 Straight
 Sites:</strong></font></p> <ul> <li><a href="http://refer.ccbill.com/cgi-bin/clicks.cgi?CA=914363-0002&PA=393415">Real
 Spankings</a></li> <li><a href="http://traffic.acpay.com/incoming.acp?progid=1035&resellerid=40105" target="_blank"><font size="2" face="Arial, Helvetica, sans-serif">Fucked
 At
 Home</font></a></li> <li><a href="http://traffic.acpay.com/incoming.acp?progid=1113&resellerid=40105" target="_blank">Hot
 Amateur
 Wives</a></li> <li><a href="http://acmoviepass.com/tour.acp?id=10031&genre=hardcore&trial=0">AC
 Movie
 Pass</a></li> </ul> <h1><font color="#000000" size="2" face="Arial, Helvetica, sans-serif">Great
 Gay
 Sites:</font></h1> <ul> <li><font size="2" face="Verdana, Arial, Helvetica, sans-serif"><a href="http://www.dudemagnet.com/">Dude
 Magnet</a></font></li> <li><font size="2" face="Verdana, Arial, Helvetica, sans-serif"><a href="http://www.gayfratguys.com/">Gay
 Frat
 Guys</a></font></li> <li><font size="2" face="Verdana, Arial, Helvetica, sans-serif"><a href="http://www.jackoff.tv/">Jackoff.tv</a></font></li> <li><font size="2" face="Verdana, Arial, Helvetica, sans-serif"><a href="http://www.seanstorm.com/">Pornstar
 -
 Sean
 Storm</a></font></li> <li><font size="2" face="Verdana, Arial, Helvetica, sans-serif"><a href="http://www.showmeguys.com/index1.shtml?">Show
 Me
 Guys</a></font></li> <li><font size="2" face="Verdana, Arial, Helvetica, sans-serif"><a href="http://www.gayvideostreams.com/">Gay
 Video
 Streams</a></font> </li> </ul> <div align="center"> <h1 align="left"><font color="#000000" size="2" face="Arial, Helvetica, sans-serif">Adult
 Search
 Engine
 Friends:</font></h1> </div> <ul> <li> <div align="left"><a href="http://www.cocklist.com/"><font size="2" face="Arial, Helvetica, sans-serif">The
 Cock
 List</font></a></div> </li> <li> <div align="left"><font size="2" face="Arial, Helvetica, sans-serif"><a href="http://www.cumseeker.com/">CumSeeker.com</a></font></div> </li> <li> <div align="left"><font size="2" face="Arial, Helvetica, sans-serif"><a href="http://www.nitrohosting.com/">Nitro
 Hosting</a></font></div> </li> <li> <div align="left"><font size="2" face="Arial, Helvetica, sans-serif"><a href="http://www.xxxpartnership.com/">XXX
 Partnership</a> </font></div> </li> <li><font size="2" face="Arial, Helvetica, sans-serif"><a href="http://www.linkmeallover.ca">Linkmeallover.ca</a></font> </li> </ul></td> <td valign="top" bgcolor="#FFFFFF"> <p align="left"><font color="#000000" size="2"><strong>Top
 Level
 Categories:</strong></font> </p> <ul> <li><font><strong><a href="http://www.wherethefuck.com/Straight_Adult_Sites/index.html">Straight
 Adult
 Sites</a></strong></font> </li> <li> <div align="left"><font><strong> <a href="http://www.wherethefuck.com/Gay_Adult_Sites/index.html">Gay
 Adult
 Sites</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Sites_For_Women/index.html">Sites
 For
 Women</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Pic_Posts/index.html">Pic
 Posts</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/TGPs/index.html">TGPs</a></strong></font></div> </li> <li> <div align="left"><font><a href="http://www.wherethefuck.com/Link_Lists_-_Top_Lists/index.html"><strong>Link
 Lists
 -
 Top
 Lists</strong></a></font></div> </li> <li><a href="http://www.wherethefuck.com/Directories_and_Search_Engines/index.html"><strong>Directories
 and
 Search
 Engines</strong></a></li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Personals/index.html">Personals</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Recreational_Sex/index.html">Recreational
 Sex</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Adult_Products/index.html">Adult
 Products</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Sexual_Health___Society/index.html">Sexual
 Health
 &amp;
 Society</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Escorts__Dancers___Massages/index.html">Escorts,
 Dancers
 &amp;
 Massages</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Business/index.html">Business</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/News/index.html">News</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Recreation/index.html">Recreation</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Chat/index.html">Chat</a></strong></font></div> </li> <li> <div align="left"><font><strong><a href="http://www.wherethefuck.com/Webmaster_Resources/index.html">Webmaster
 Resources</a></strong></font></div> </li> </ul></td> </tr> </table></td> </tr> <tr> <td colspan=3 valign="top"> <img src="http://www.wherethefuck.com/images/bottom.gif" alt=""></td> </tr> </table></td> </tr> </table></td> </tr> <tr> <td colspan="2"> <div align="center"><font color="#FFFFFF"><small> <a href="http://www.wherethefuck.com/cgi-bin/gforums/gforum.cgi?"> <b>&nbsp;SUPPORT FORUM</b></a><b>&nbsp; <img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Enginer"> <a href="http://www.wherethefuck.com/New"> &nbsp;NEW LINKS</a> &nbsp;<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a href="http://www.wherethefuck.com/Cool"> &nbsp;COOL LINKS</a> &nbsp;<img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a href="http://www.wherethefuck.com/Ratings"> &nbsp;TOP RATED</a>&nbsp; <img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a href="http://www.wherethefuck.com/cgi-bin/jump.cgi?ID=random&d=1">&nbsp;RANDOM LINK</a>&nbsp; <img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a href="http://www.wherethefuck.com/cgi-bin/search.cgi">
 &nbsp;SEARCH &nbsp; </a><img src="http://www.wherethefuck.com/images/mint/dot.gif" alt="Where The Fuck Adult Search Engine"> <a href="http://www.wherethefuck.com/cgi-bin/user.cgi"> &nbsp;LOGIN</a>&nbsp;
</b> </small></font></div> <div align="center"><font size="1" face="Verdana, Arial, Helvetica, sans-serif">2002
 Copyright, South Beach Media Group, Inc.</font></div></td> </tr> </table> <td>&nbsp;</body></html>
    porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole
    porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole
    porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole
    porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole
    porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole porn fuck sex asshole
    
    """)
