# -*- coding: utf-8 -*-
"""utilitaires divers (parsing html, etc)

nettoyage des chaines de caractères

- beautiful_soup() pour le html
- filter_meta() pour les meta
- normalize_string() pour les chaines simples 

"""

###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################


from HTMLParser import HTMLParser
import re
import unicodedata as U
import string
from urlparse import urlparse

from filtrage.BeautifulSoup import BeautifulSoup, SoupStrainer, Comment
from filtrage.config import META_WEIGHT, BODY_WEIGHT


def normalize_string(punctuated):
    """supprime les caractères de ponctuation dans un texte
    et met la chaine en minuscule
    """
    rgx = re.compile('[%s]' % string.punctuation)
    # minuscule
    punctuated.lower()
    # extraction de la ponctuation
    unpunctuated = re.sub(rgx, " ", punctuated)
    # suppressin des accents
    return un_accentuation(unpunctuated)

class MetaTagParser(HTMLParser):
    """parseur souple pour extraire les balises meta dans une page html
    """
    def __init__(self):
        self.content = []
        HTMLParser.__init__(self)
        
    def handle_starttag(self, tag, attrs):
        attrs = dict((key.lower(), value.lower()) for key, value in attrs)
        # parsing suivant la balise meta
        # self.current_tag = tag.lower()
        if tag.lower() == 'meta':
            if attrs.get('name') in ('description', 'keywords'):
                content = attrs.get('content')
                if content:
                    # enlevons la ponctuation, etc.
                    self.content.append(normalize_string(content))

def filter_meta(data):
    """parse les metatags, extrait les mots clefs"""
    parser = MetaTagParser()
    try:
        parser.feed(data)
    except:
        pass

    return " ".join(parser.content)

def get_hostname(url):
    """recupère le nom de domaine d'une url"""
    url = urlparse(url)
    return url.hostname

def url_filter_domain(url):
    "url sans http:// et sans www, c'est-à-dire uniquement le nom de domaine"""
    url = get_hostname(url)
#     if url.startswith("http://"):
#         url = url.replace('http://', '', 1)
    if url.startswith('www.'):
        url = url.replace('www.', '', 1)
    if url.endswith('/'):
        url = url[0:-1]        
    return url

def url_add_protocol(url):
    "rajoute le protocole à une url"
    # FIXME : utiliser urlparse (c'est plus propre)
    # if urlparser(url).protocol = "": 
    if url.startswith("http://"):
        return url
    else:
        return "http://" + url.strip()

# def get_url_keywords(url, keywords):
#     """permet de repérer si l'url contient des mot-clefs de la blacklist
#     keywords: liste des mots-clef de la blacklist
#     """
#     # recupere le nom de domaine
#     url = get_hostname(url)
#     # s'il y a des points dans l'adresse
#     words = url.split(".")
#     for word in words:
#         if keyword in keywords:
#             yield keyword
#     # s'il y des tirets dans l'adresse
#     words = url.split("-")
#     for word in words:
#         if keyword in keywords:
#             yield keyword
        
def un_accentuation(string):
    """permet de supprimer les accents,
    quelque soit le langage
    exemple éèê -> e
    """
    if not type(string) == unicode:
        unicode_string = unicode(string, "utf-8")
    else:
        unicode_string = string
    result = ''.join(U.normalize('NFD', x)[0] for x in unicode_string)
    # renvoie une string
    return result.encode('utf-8')

    # return  ''.join(U.normalize('NFD', x)[0] for x in unicode_string)
# def get_urls(html):
#     """parse le contenu d'une page, et récupère les liens
#     page : contenu d'une page, exemple : 
#     [<a href="http://www.bob.com/">success</a>, 
#      <a href="http://www.bob.com/plasma">experiments</a>, 
#      <a href="http://www.boogabooga.net/">BoogaBooga</a>]
#     """
#     # récuperation des urls
#     links = SoupStrainer('a')
#     l = [tag for tag in BeautifulSoup(html, parseOnlyThese=links)]
#     return l

def get_content(html):
    """page : contenu d'une page
    renvoit le corps de texte (body) sans les commentaires 
    """
    # corps de texte
    soup = BeautifulSoup(html).body
    if soup == None:
        soup = BeautifulSoup(html)
        
    # extraction des commentaires
    comments = soup.findAll(text=lambda text:isinstance(text, Comment))
    [comment.extract() for comment in comments]
    return str(soup)

def strip_tags(html):
    """suppression des balises"""
    result = []
    parser = HTMLParser()
    parser.handle_data = result.append
    try:
        parser.feed(html)
        parser.close()
    except:
        pass
    
    return ''.join(result)

def beautiful_soup(html):
    """netoyage du html"""
    html = get_content(html)
    text = strip_tags(html)
    return normalize_string(text)

## calculs de moyenne pondérée

def lst_to_dct(rates):
    """affichage des résultats sous forme de dictionnaire
    permet de passer de 
    [('porn', 0.94649927972169501), ('noporn', 0.067599721064875107)]
    [('noporn', 0.99990000000000001)]
    à un format :
    {'porn': 0.94649927972169501}
    """
    d = {}
    for name, rate in rates:
        d[name]=rate 
    return d

def dct_to_lst(rates):
    result = []
    for name, rate in rates.items():
        result.append((name, rate))
    return result

def average(meta=[], body=[]):
    """moyenne arithmetique pour l'évaluation pondérée
    exemple de traitement :
    args : sont des listes de ponderations
    [('porn', 0.98215047377065301), ('noporn', 0.045134455898390435)]

    """
    sum_weight = META_WEIGHT + BODY_WEIGHT

    meta_rates = lst_to_dct(meta)
    body_rates = lst_to_dct(body)

    # determinons tous les mots-clef present
    merge_keywords = {}
    merge_keywords.update(meta_rates)
    merge_keywords.update(body_rates)
    keywords = merge_keywords.keys()
    average = {}
    for key in keywords:
        if ( key in meta_rates.keys() ) and ( key in body_rates.keys() ):
            rate = META_WEIGHT * meta_rates[key] + BODY_WEIGHT * body_rates[key]
            average[key] = rate / sum_weight
    # les autres valeurs ne changent pas 
    merge_keywords.update(average)
    return dct_to_lst(merge_keywords)
